package Sandbox::Resolver;
use strict;
use warnings;
use base 'Class::Data::Inheritable';

use Apache2::Connection ();
use Apache2::Filter ();
use APR::Bucket ();
use APR::Brigade ();
use APR::Util;

use APR::Const -compile => qw(SUCCESS);
use Apache2::Const -compile => qw(OK MODE_GETLINE);

use Net::DNS;

__PACKAGE__->mk_classdata(resolver => Net::DNS::Resolver->new);

sub handler : method {
    my ($class, $c) = @_;
    my $bb_in  = APR::Brigade->new($c->pool, $c->bucket_alloc);
    my $bb_out = APR::Brigade->new($c->pool, $c->bucket_alloc);
    while (1) {
        $c->input_filters->get_brigade($bb_in, Apache2::Const::MODE_GETLINE);
        my $data = $class->read_from_brigade($bb_in);
        last unless $data;
        $bb_out->insert_tail(APR::Bucket->new($c->bucket_alloc, $class->search($data)));
        $bb_out->insert_tail(APR::Bucket::flush_create($c->bucket_alloc));
        $c->output_filters->pass_brigade($bb_out);
    }
    return Apache2::Const::OK;
}

sub read_from_brigade {
    my ($class, $brigade) = @_;
    my $buff;
    while (!$brigade->is_empty) {
        my $bucket = $brigade->first;
        return if ($bucket->is_eos);
        $bucket->read($buff);
        $bucket->remove;
    }
    $buff =~ s/[\r\n]*$//; # telnetб
    return $buff;
}

sub search {
    my ($class, $input) = @_;
    my $query = $class->resolver->search($input);
    $query ? join("\n", map { $_->address } $query->answer) . "\n"
           : sprintf "error: %s\n", $class->resolver->errorstring;
}

1;
